/*-----------------------------------------------------------------------------
 *
 *  Kaiser's Strife Script Compiler version 3
 *  A compiling utility for the Doom engine game, Strife.
 *  2/3/07 Samuel Villarreal
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * DESCRIPTION:
 *     File handing
 *     File opening/saving. NPC block saving/parsing
 *
 * Author: Kaiser
 *
 *--------------------------------------------------------------------*/

#include "localRef.h"

//______________________________________________________________________
//RogueWriter::RogueWriter
//______________________________________________________________________

RogueWriter::RogueWriter(const char *file) {
	this->buf = fopen(file, "wb");
}

//______________________________________________________________________
//RogueWriter::~RogueWriter
//______________________________________________________________________

RogueWriter::~RogueWriter() {
	fclose(this->buf);
}

//______________________________________________________________________
//RogueWriter::WriteBlock: Writes per NPC block to binary file
//______________________________________________________________________

void RogueWriter::WriteBlock(const npc_t* x) {
	fwrite(x, BLOCKSIZE, 1, this->buf);
}

//______________________________________________________________________
//FParse::FParse: File input and output names are setup and initialized
//______________________________________________________________________

FParse::FParse(const char* file, bool read, const char* outFile) {
	FILE* f = fopen(file, "rb");

	if(!f)
		kerr("FParse: No file found..");

	strncpy(filePath.fileInput, file, FILELEN);

	fseek(f, 0, SEEK_END);
	filePath.inputSize = ftell(f);
	fseek(f, 0, SEEK_SET);

	stream = new byte[filePath.inputSize];
	if(read)fread(stream, filePath.inputSize, 1, f);
	else memset(stream, 0, filePath.inputSize);

	F_GetFileName(file, filePath.inputName);
	F_GetPath(file, filePath.path);

	this->F_RemoveExt(this->filePath.fileInput, this->filePath.fileOutput);
	strcat(this->filePath.fileOutput, outFile);

	fclose(f);
}

//______________________________________________________________________
//FParse::~FParse
//______________________________________________________________________

FParse::~FParse() {
	delete [] this->stream;

	if(this->nStream[0])
		delete [] this->nStream;
}

//______________________________________________________________________
//FParse::F_ZapComments
//______________________________________________________________________

void FParse::F_ZapComments(const int size) {
	bool foundComment = false;
	int nStrBuf = 0;

	this->nStream = new byte[size];

	for( int i = 0; i < size; i++ ) {
		if(this->stream[i] == '#') {
			foundComment = true;
		}

		if(foundComment == false)
			this->nStream[nStrBuf++] = this->stream[i];

		if(this->stream[i] == '\n')
			foundComment = false;
	}

	if(nStrBuf != size)
		this->filePath.inputSize = size - (size - nStrBuf);
}

//______________________________________________________________________
//FParse::F_GetPath
//______________________________________________________________________

void FParse::F_GetPath(const char* name, char* out) {
	int pos = 0;

	pos = strlen(name)-1;

	while( pos && (name[pos] != FOWARDSLASH && name[pos] != BACKSLASH ) ) pos--;
	
	if(pos <= 0)
		kerr("F_GetPath: Invalid path: %s..", name);

	memcpy(out, name, pos+1);
}

//______________________________________________________________________
//FParse::F_GetFileName
//______________________________________________________________________

void FParse::F_GetFileName(const char* name, char* out) {
	int len = strlen(name)-1;
	int pos = 0;

	for( int i = 0; i < len; i++ ) {
		if(name[i] == FOWARDSLASH || name[i] == BACKSLASH)
			pos = i;
	}

	if((len - pos) <= 0)
		kerr("F_GetFileName: %s has no file..", name);

	memcpy(out, &name[pos+1], len - pos);
}

//______________________________________________________________________
//FParse::F_RemoveExt
//______________________________________________________________________

void FParse::F_RemoveExt(const char* name, char* out) {
	int pos = strlen(name)-1;

	while( pos && name[pos] != '.' ) pos--;

	if(pos <= 0)
		kerr("F_RemoveExt: No file extention in input: %s..", name);

	memcpy(out, name, pos);
}

/*file.cpp eof*/