/*-----------------------------------------------------------------------------
 *
 *  Kaiser's Strife Script Compiler version 3
 *  A compiling utility for the Doom engine game, Strife.
 *  2/3/07 Samuel Villarreal
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * DESCRIPTION:
 *     Main code
 *     Program initialization
 *
 * Author: Kaiser
 *
 *--------------------------------------------------------------------*/

#include "localRef.h"
#include "tkParse.h"

npc_t npc[MAX_NPC];

//______________________________________________________________________
//Splash
//______________________________________________________________________

static void Splash(void) {
	kput("\t\t****************************************");
	kput("\t\t** Kaiser's Strife Script Compiler 3  **");
	kput("\t\t**    email: svkaiser@gmail.com       **");
	kput("\t\t****************************************");
	kput("\n\n");
	kput("Usage: <kssc.exe> <scriptfile> <option>\n");
	kput("Options:\n-d\t\tDecompile script lump");
	kput("-decompile\tDecompile script lump\n\n");
}

//______________________________________________________________________
//CheckBlock: Binary lumps must be in multiples of 1516 as
// it must match the struct size
//______________________________________________________________________

static inline bool CheckBlock(int size) {
	if(size % BLOCKSIZE != 0)
		return false;

	return true;
}

//______________________________________________________________________
//GetBlocks: Get the number of NPC blocks
//______________________________________________________________________

static inline int GetBlocks(int size) {
	return size / BLOCKSIZE;
}

//______________________________________________________________________
//ResetBlocks: Clear the char arrays for any garbage during initialization
//______________________________________________________________________

static inline void ResetBlocks(void) {
	for( int i = 0; i < MAX_NPC; i++ ) {
		memset(npc[i].Name, 0, sizeof(npc[i].Name));
		memset(npc[i].Backdrop, 0, sizeof(npc[i].Backdrop));
		memset(npc[i].Sound, 0, sizeof(npc[i].Sound));
		memset(npc[i].Dialogue, 0, sizeof(npc[i].Dialogue));

		for( int c = 0; c < 5; c++ ) {
			memset(npc[i].Responses[c].Reply, 0,
				sizeof(npc[i].Responses[c].Reply));
			memset(npc[i].Responses[c].No, 0,
				sizeof(npc[i].Responses[c].No));
			memset(npc[i].Responses[c].Yes, 0,
				sizeof(npc[i].Responses[c].Yes));
		}
	}
}

//______________________________________________________________________
//main: Play splash title and check for either decompile or compile options
//______________________________________________________________________

int main (int argc, char**argv) {

	Splash();

	/**************************Decompile Mode*****************************/
	if(argv[2]) {
		if(strcmp(argv[2], DECOMP1) && strcmp(argv[2], DECOMP2))
			kerr("Main: Invalid argument: %s", argv[2]);

		static FParse *Parse = new FParse(argv[1], 1, ".txt");
		kput("Loaded %s at %i bytes", Parse->filePath.fileInput,
			Parse->filePath.inputSize);

		kput("Decompiling: %s", Parse->filePath.inputName);

		static int blockNum = 0;
		static int curType;
		static const int numNPC = GetBlocks(Parse->filePath.inputSize);

		kput("%i blocks loaded::", numNPC);

		if(CheckBlock(Parse->filePath.inputSize) == false)
			kerr("Main: Script size is not valid!");

		memcpy(npc, Parse->stream, Parse->filePath.inputSize);

		curType = npc[0].SpeakerType;

		FILE* f = fopen(Parse->filePath.fileOutput, "w");
		for( int i = 0; i < numNPC; i++ ) {

			if(curType == npc[i].SpeakerType)
				blockNum++;
			else {
				blockNum = 1;
				curType = npc[i].SpeakerType;
			}

			cput(f, "__________%s Block %i____________", npc[i].Name, blockNum);
			fprintf(f, "ACTOR\t%i\n", npc[i].SpeakerType);
			fprintf(f, "\tDROP\t\t%i\n", npc[i].DropType);

			if(npc[i].Name[0])
				fprintf(f, "\tNAME\t\t%c%s%c\n",
					ASCII_QUOTE, 
					kchk(sizeof(npc[i].Name), npc[i].Name),
					ASCII_QUOTE);

			fprintf(f, "\tIF_ITEM\t\t%i\n", npc[i].ItemCheck[0]);
			fprintf(f, "\tIF_ITEM2\t\t%i\n", npc[i].ItemCheck[1]);
			fprintf(f, "\tIF_ITEM3\t\t%i\n", npc[i].ItemCheck[2]);
			fprintf(f, "\tGOTO\t\t%i\n", npc[i].Link);

			if(npc[i].Sound[0])
				fprintf(f, "\tVOICE\t\t%c%s%c\n",
					ASCII_QUOTE,
					kchk(sizeof(npc[i].Sound), npc[i].Sound),
					ASCII_QUOTE);

			if(npc[i].Backdrop[0])
				fprintf(f, "\tPANEL\t\t%c%s%c\n",
					ASCII_QUOTE,
					kchk(sizeof(npc[i].Backdrop), npc[i].Backdrop),
					ASCII_QUOTE);

			if(npc[i].Dialogue[0])
				fprintf(f, "\tDIALOG\t\t%c%s%c\n\n",
					ASCII_QUOTE, 
					kchk(sizeof(npc[i].Dialogue), npc[i].Dialogue), 
					ASCII_QUOTE);

			for( int k = 0; k < MAX_CHOICE; k++ ) {
				if(!(npc[i].Responses[k].Reply[0] == '\0')) {
					fprintf(f, "\tCHOICE\n");

					if(npc[i].Responses[k].Reply[0])
						fprintf(f, "\t\tTEXT\t\t%c%s%c\n",
							ASCII_QUOTE, 
							kchk(sizeof(npc[i].Responses[k].Reply),
							npc[i].Responses[k].Reply), 
							ASCII_QUOTE);

					fprintf(f, "\t\tGIVE\t\t%i\n", npc[i].Responses[k].GiveType);
					fprintf(f, "\t\tLINK\t\t%i\n", npc[i].Responses[k].Link);

					if(npc[i].Responses[k].Yes)
						fprintf(f, "\t\tYES\t\t%c%s%c\n",
							ASCII_QUOTE, 
							kchk(sizeof(npc[i].Responses[k].Yes),
							npc[i].Responses[k].Yes), 
							ASCII_QUOTE);

					if(npc[i].Responses[k].No)
						fprintf(f, "\t\tNO\t\t%c%s%c\n",
							ASCII_QUOTE, 
							kchk(sizeof(npc[i].Responses[k].No),
							npc[i].Responses[k].No), 
							ASCII_QUOTE);

					fprintf(f, "\t\tLOG\t\t%i\n", npc[i].Responses[k].Log);

					if(npc[i].Responses[k].Item[1]) {
						fprintf(f, "\t\tNODISPLAYCOST\t%i * %i\n",
							npc[i].Responses[k].Item[1],
							npc[i].Responses[k].Count[1]);
					}
					else
						fprintf(f, "\t\tCOST\t\t%i * %i\n",
							npc[i].Responses[k].Item[0],
							npc[i].Responses[k].Count[0]);

					fprintf(f, "\t\tALTCOST\t\t%i * %i\n",
						npc[i].Responses[k].Item[2],
						npc[i].Responses[k].Count[2]);

					fprintf(f, "\t;\n\n");
				}
			}
			fprintf(f, "END\n\n");
		}

		kput("Saving %s...", Parse->filePath.fileOutput);

		fclose(f);

		delete Parse;
	}
	/**************************End Decompile Mode**************************/



	/*****************************Compile Mode*****************************/
	else {
		static FParse *Parse = new FParse(argv[1], 1, ".lmp");
		kput("Loaded %s at %i bytes", Parse->filePath.fileInput,
			Parse->filePath.inputSize);

		kput("Compiling: %s", Parse->filePath.inputName);
		ResetBlocks();

		Parse->F_ZapComments(Parse->filePath.inputSize);

		tk_ParseScript(Parse->nStream, Parse->filePath.inputSize);

		kput("Saving %s...", Parse->filePath.fileOutput);
		delete Parse;

		static RogueWriter *r = new RogueWriter(Parse->filePath.fileOutput);

		for( int i = 0; npc[i].SpeakerType != 0; i++ ) {
			r->WriteBlock(&npc[i]);
		}

		delete r;
	}
	/***************************End Compile Mode***************************/

	return 1;
}

/*kssc.cpp eof*/